@@ -3,12 +3,12 @@
  *      Guido Draheim <guidod@gmx.de>
  *      Tomi Ollila <Tomi.Ollila@iki.fi>
  *
- * Copyright (c) 1999,2000,2001,2002 Guido Draheim
+ * Copyright (c) 1999,2000,2001,2002,2003 Guido Draheim
  *          All rights reserved,
  *          use under the restrictions of the
  *          Lesser GNU General Public License
- *          note the additional license information 
- *          that can be found in COPYING.ZZIP
+ *          or alternatively the restrictions 
+ *          of the Mozilla Public License 1.1
  */
 
 #include <zzip/lib.h>                                         /* exported...*/
@@ -21,6 +21,7 @@
 #include <ctype.h>
 
 #include <zzip/format.h>
+#include <zzip/fetch.h>
 #include <zzip/__debug.h>
 
 #if 0
@@ -48,7 +49,6 @@ zzip_file_close(ZZIP_FILE * fp)
     ZZIP_DIR * dir = fp->dir;
     
     if (fp->method)
-
         inflateEnd(&fp->d_stream); /* inflateEnd() can be called many times */
 
     if (fp->buf32k)
@@ -77,7 +77,7 @@ zzip_file_saveoffset(ZZIP_FILE * fp)
     if (fp)
     {
         int fd = fp->dir->fd;
-        zzip_off_t off = fp->io->seeks(fd, 0, SEEK_CUR);
+        zzip_off_t off = fp->io->fd.seeks(fd, 0, SEEK_CUR);
         if (off < 0)
             return -1;
 
@@ -119,7 +119,7 @@ _dirsep_casecmp (zzip_char_t* s1, zzip_char_t* s2)
 	c1 = (int)(unsigned char) *s1;
 	c2 = (int)(unsigned char) *s2;
 	if ((c1&0xE0) == 0x40) c1 = mapping[c1&0x1f];
-	if ((c1&0xE0) == 0x40) c2 = mapping[c2&0x1f];
+	if ((c2&0xE0) == 0x40) c2 = mapping[c2&0x1f];
 	if (c1 != c2)
 	    return (c1 - c2);
 	s1++; s2++;
@@ -215,7 +215,7 @@ zzip_file_open(ZZIP_DIR * dir, zzip_char_t* name, int o_mode)
             fp->offset = hdr->d_off;
             dir->currentfp = fp;
 
-            if (dir->io->seeks(dir->fd, hdr->d_off, SEEK_SET) < 0)
+            if (dir->io->fd.seeks(dir->fd, hdr->d_off, SEEK_SET) < 0)
                 { err = ZZIP_DIR_SEEK; goto error; }
 
             {   /* skip local header - should test tons of other info, 
@@ -223,18 +223,18 @@ zzip_file_open(ZZIP_DIR * dir, zzip_char_t* name, int o_mode)
                 zzip_ssize_t dataoff;
                 struct zzip_file_header * p = (void*) fp->buf32k;
 
-		dataoff = dir->io->read(dir->fd, (void*)p, sizeof(*p));
+		dataoff = dir->io->fd.read(dir->fd, (void*)p, sizeof(*p));
 		if (dataoff < (zzip_ssize_t)sizeof(*p))
 		{ err = ZZIP_DIR_READ;  goto error; }
-                if (! ZZIP_FILE_HEADER_CHECKMAGIC(p)) /* PK\3\4 */
+                if (! zzip_file_header_check_magic(p)) /* PK\3\4 */
 		{ err = ZZIP_CORRUPTED; goto error; }
 
-                dataoff = ZZIP_GET16(p->z_namlen) + ZZIP_GET16(p->z_extras);
+                dataoff = zzip_file_header_sizeof_tail(p);
               
-                if (dir->io->seeks(dir->fd, dataoff, SEEK_CUR) < 0)
+                if (dir->io->fd.seeks(dir->fd, dataoff, SEEK_CUR) < 0)
                 { err = ZZIP_DIR_SEEK; goto error; }
 
-                fp->dataoffset = dir->io->tells(dir->fd);
+                fp->dataoffset = dir->io->fd.tells(dir->fd);
                 fp->usize = hdr->d_usize;
                 fp->csize = hdr->d_csize;
             }
@@ -295,7 +295,7 @@ zzip_fclose(ZZIP_FILE * fp)
 {
     if (! fp) return 0;
     if (! fp->dir) 
-      { int r = fp->io->close(fp->fd); free(fp); return r; } /* stat fd */
+      { int r = fp->io->fd.close(fp->fd); free(fp); return r; } /* stat fd */
     else return zzip_file_close(fp);
 }
 
@@ -342,7 +342,7 @@ zzip_file_read(ZZIP_FILE * fp, char * buf, zzip_size_t len)
      if (dir->currentfp != fp)
      {
          if (zzip_file_saveoffset(dir->currentfp) < 0 
-         || fp->io->seeks(dir->fd, fp->offset, SEEK_SET) < 0)
+         || fp->io->fd.seeks(dir->fd, fp->offset, SEEK_SET) < 0)
            { dir->errcode = ZZIP_DIR_SEEK; return -1; }
          else
            { dir->currentfp = fp; }
@@ -364,7 +364,7 @@ zzip_file_read(ZZIP_FILE * fp, char * buf, zzip_size_t len)
 				    fp->crestlen : ZZIP_32K );
              /*  zzip_size_t cl = fp->crestlen > 128 ? 128 : fp->crestlen; */
 
-                 zzip_ssize_t i = fp->io->read(dir->fd, fp->buf32k, cl);
+                 zzip_ssize_t i = fp->io->fd.read(dir->fd, fp->buf32k, cl);
                  if (i <= 0)
                  {
                      dir->errcode = ZZIP_DIR_READ; /* or ZZIP_DIR_READ_EOF ? */
@@ -390,7 +390,7 @@ zzip_file_read(ZZIP_FILE * fp, char * buf, zzip_size_t len)
          return l - fp->d_stream.avail_out;
      }else
      {   /* method == 0 -- unstore */
-         rv = fp->io->read(dir->fd, buf, l);
+         rv = fp->io->fd.read(dir->fd, buf, l);
          if (rv > 0)
              { fp->restlen-= rv; }
          else 
@@ -416,7 +416,7 @@ zzip_read(ZZIP_FILE * fp, char * buf, zzip_size_t len)
 {
     if (! fp) return 0;
     if (! fp->dir) 
-      { return fp->io->read(fp->fd, buf, len); } /* stat fd */
+      { return fp->io->fd.read(fp->fd, buf, len); } /* stat fd */
     else
     {   register zzip_ssize_t v;
         v = zzip_file_read(fp, buf, len);
@@ -681,11 +681,11 @@ zzip_open_shared_io (ZZIP_FILE* stream,
     {   
 	zzip_plugin_io_t os = (o_modes & ZZIP_ALLOWREAL)
 	    ?  zzip_get_default_io () : io;
-	int fd = os->open(filename, o_flags); /* io->open */
+	int fd = os->fd.open(filename, o_flags); /* io->fd.open */
         if (fd != -1)
         {
             ZZIP_FILE* fp = calloc (1, sizeof(ZZIP_FILE));
-            if (!fp) { os->close(fd); return 0; } /* io->close */
+            if (!fp) { os->fd.close(fd); return 0; } /* io->fd.close */
 
             fp->fd = fd; 
             fp->io = os;
@@ -738,7 +738,7 @@ zzip_open_shared_io (ZZIP_FILE* stream,
 /*    found: */
           /* found zip-file, now try to parse it */
           dir = zzip_dir_fdopen_ext_io(fd, &e, ext, io);
-          if (e) { errno = zzip_errno(e); io->close(fd); return 0; }
+          if (e) { errno = zzip_errno(e); io->fd.close(fd); return 0; }
 
           /* (p - basename) is the lenghtof zzip_dir part of the filename */
           fp = zzip_file_open(dir, filename + (p - basename) +1, o_modes);
@@ -762,6 +762,19 @@ zzip_open_shared_io (ZZIP_FILE* stream,
 #undef zzip_open_ext_io    /* zzip_open_ext_io64 */
 #undef zzip_opendir_ext_io /* zzip_opendir_ext_io64 */
 
+_zzip_export
+ZZIP_FILE * zzip_open_shared_io(ZZIP_FILE* stream,
+				zzip_char_t* name, int o_flags, int o_modes,
+				zzip_strings_t* ext, zzip_plugin_io_t io);
+_zzip_export
+ZZIP_FILE * zzip_open_ext_io(zzip_char_t* name, int o_flags, int o_modes,
+			     zzip_strings_t* ext, zzip_plugin_io_t io);
+_zzip_export
+ZZIP_DIR *  zzip_opendir_ext_io(zzip_char_t* name, int o_modes,
+				zzip_strings_t* ext, zzip_plugin_io_t io);
+
+/* DLL compatibility layer - so that 32bit code can link with this lib too */
+
 _zzip_export
 ZZIP_FILE * zzip_open_shared_io(ZZIP_FILE* stream,
 				zzip_char_t* name, int o_flags, int o_modes,
@@ -809,7 +822,7 @@ zzip_rewind(ZZIP_FILE *fp)
 
     if (! fp->dir) 
     { /* stat fd */
-        fp->io->seeks(fp->fd,0,SEEK_SET);
+        fp->io->fd.seeks(fp->fd,0,SEEK_SET);
         return 0;
     }
     
@@ -826,7 +839,7 @@ zzip_rewind(ZZIP_FILE *fp)
     }
     
     /* seek to beginning of this file */
-    if (fp->io->seeks(dir->fd, fp->dataoffset, SEEK_SET) < 0)
+    if (fp->io->fd.seeks(dir->fd, fp->dataoffset, SEEK_SET) < 0)
         return -1;
     
     /* reset the inflate init stuff */
@@ -835,10 +848,11 @@ zzip_rewind(ZZIP_FILE *fp)
     
     if (fp->method) 
     { /* method == 8, deflate */
-        memset(&fp->d_stream, 0, sizeof fp->d_stream);
-        err = inflateInit2(&fp->d_stream, -MAX_WBITS);
+        err = inflateReset(&fp->d_stream);
         if (err != Z_OK) { goto error; }
-        
+
+	/* start over at next inflate with a fresh read() */
+	fp->d_stream.avail_in = 0;
         fp->crestlen = fp->csize;
     }
 
@@ -874,7 +888,7 @@ zzip_seek(ZZIP_FILE * fp, zzip_off_t offset, int whence)
 
     if (! fp->dir) 
     { /* stat fd */
-        return fp->io->seeks(fp->fd, offset, whence);
+        return fp->io->fd.seeks(fp->fd, offset, whence);
     }
 
     cur_pos = zzip_tell(fp);
@@ -927,7 +941,7 @@ zzip_seek(ZZIP_FILE * fp, zzip_off_t offset, int whence)
     if (dir->currentfp != fp)  
     {
         if (zzip_file_saveoffset(dir->currentfp) < 0 
-            || dir->currentfp->io->seeks(dir->fd, fp->offset, SEEK_SET) < 0)
+            || dir->currentfp->io->fd.seeks(dir->fd, fp->offset, SEEK_SET) < 0)
         { dir->errcode = ZZIP_DIR_SEEK; return -1; }
         else
         { dir->currentfp = fp; }
@@ -935,8 +949,8 @@ zzip_seek(ZZIP_FILE * fp, zzip_off_t offset, int whence)
   
     if (fp->method == 0) 
     { /* unstore, just lseek relatively */
-        ofs = fp->io->tells(dir->fd);
-        ofs = fp->io->seeks(dir->fd,read_size,SEEK_CUR);
+        ofs = fp->io->fd.tells(dir->fd);
+        ofs = fp->io->fd.seeks(dir->fd,read_size,SEEK_CUR);
         if (ofs > 0) 
         { /* readjust from beginning of file */
             ofs -= fp->dataoffset;
@@ -985,7 +999,7 @@ zzip_tell(ZZIP_FILE * fp)
         return -1;
 
     if (! fp->dir)  /* stat fd */
-        return fp->io->tells(fp->fd);
+        return fp->io->fd.tells(fp->fd);
 
     /* current uncompressed offset is uncompressed size - data left */
     return (fp->usize - fp->restlen);
